// Copyright (c) 2004-2005 Lutz Latta
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.

#ifndef ParticleSystemConfig_H
#define ParticleSystemConfig_H

#include <string>
using namespace std;

#include "ParticleSystem.h"

class ParticleSystemConfig
{

public:

	ParticleSystemConfig(int& width, int& height,
			vector<ParticleEmitterData>& emitters, vector<ParticleAttributesType>& particleAttributes,
			string& diffuseTextureName, vector<float3>& localForcePositions,
			string& flowFieldTextureName, float3& flowFieldScale,
			float3& sphereColliderPosition,
			float3& viewerPosition, float2& viewerRotation, float4& clearColor, 
			ParticleSystem::BlendFunctionType& blendFunction, bool& doSorting,
			bool menuKeys[256])
		: Width(width), Height(height), Emitters(emitters),
		ParticleAttributes(particleAttributes), DiffuseTextureName(diffuseTextureName),
		LocalForcePositions(localForcePositions),
		FlowFieldTextureName(flowFieldTextureName), FlowFieldScale(flowFieldScale),
		SphereColliderPosition(sphereColliderPosition), 
		ViewerPosition(viewerPosition), ViewerRotation(viewerRotation), ClearColor(clearColor),
		BlendFunction(blendFunction), DoSorting(doSorting), MenuKeys(menuKeys),
		ParserState(PS_MAIN), CurrentKeyframeIndex(0) {}

	void ReadConfig(const string& fileName);

	int& Width;
	int& Height;

	vector<ParticleEmitterData>& Emitters;
	vector<ParticleAttributesType>& ParticleAttributes;

	string& DiffuseTextureName;

	vector<float3>& LocalForcePositions;

	string& FlowFieldTextureName;
	float3& FlowFieldScale;

	float3& SphereColliderPosition;

	float3& ViewerPosition;
	float2& ViewerRotation;
	float4& ClearColor;
	ParticleSystem::BlendFunctionType& BlendFunction;
	bool& DoSorting;

	bool* MenuKeys;

private:

	static void startElement(void *userData, const char *name, const char **attributes);
	static void endElement(void *userData, const char *name);


	float CurrentKeyframeTimes[4];
	float4 CurrentKeyframes[4];
	int CurrentKeyframeIndex;

	enum ParserStateType
	{
		PS_MAIN,
		PS_EMITTER,
		PS_ATTRIBUTES
	} ParserState;

};

#endif
