/*
  Render to vertex array class
  sgreen 12/2002
  
  - allocates array in video memory for VAR/PDR
  - does ReadPixels from float pbuffer to VAR memory using PDR
  - in theory this can happen entirely on the card, without any AGP transfers 

  to do:
  generalize
  use fences
*/

#ifndef RenderToVA_H
#define RenderToVA_H

#ifdef _WIN32
 #include <windows.h>
 #include <GL/wglext.h>
#else
 #include <GL/glxext.h>
 #define wglAllocateMemoryNV glXAllocateMemoryNV
 #define wglFreeMemoryNV glXFreeMemoryNV
#endif

#include <GL/gl.h>
#include <GL/glext.h>

#include "shared/RenderTexture.h"

class RenderableVertexArray {
public:
	RenderableVertexArray(int nverts, int nattribs)
		: m_nverts(nverts), m_nattribs(nattribs)
	{
		m_size = m_nverts * m_nattribs * sizeof(float);

		// allocate PDR/VAR memory
//		m_data = (char *) wglAllocateMemoryNV(m_size, 0.0, 0.0, 1.0);  // this should give video mem
//		m_data = (char *) wglAllocateMemoryNV(m_size, 1.0, 0.0, 1.0);  // this should give video mem
		m_data = (char *) wglAllocateMemoryNV(m_size, 0.0, 1.0, 1.0);  // this should give video mem
//		m_data = (char *) wglAllocateMemoryNV(m_size, 0.0, 0.0, 0.5);  // this should give agp mem
		if (!m_data) {
			fprintf(stderr, "Error allocating video memory\n");
			return;
		}

		//glPrimitiveRestartIndexNV(0xfff);
	}

	~RenderableVertexArray()
	{
		wglFreeMemoryNV(m_data);
	}

	float* GetPointerBehindPositions()
	{
		return (float*)(m_data) + m_nverts * 3;
	}

	// set vertex array pointers
	void SetPointers()
	{
		glVertexPointer(3, GL_FLOAT, 0, m_data);
		glEnableClientState(GL_VERTEX_ARRAY);

		//glTexCoordPointer(2, GL_FLOAT, 0, m_data + m_nverts * 3 * sizeof(float));
		//glEnableClientState(GL_TEXTURE_COORD_ARRAY);

		// create vertex array range
		glVertexArrayRangeNV(m_size, m_data);
		glEnableClientState(GL_VERTEX_ARRAY_RANGE_NV);

		// create pixel data range
		glPixelDataRangeNV(GL_READ_PIXEL_DATA_RANGE_NV, m_size, m_data);
		glEnableClientState(GL_READ_PIXEL_DATA_RANGE_NV);

		GLboolean valid;
		glGetBooleanv(GL_VERTEX_ARRAY_RANGE_VALID_NV, &valid);
		assert(valid);
	}

	// read data from float pbuffer to position channel of vertex array 
	void Read(RenderTexture* pbuffer)
	{
		assert(m_nverts == pbuffer->GetWidth() * pbuffer->GetHeight());
		pbuffer->BeginCapture();
		glReadPixels(0, 0, pbuffer->GetWidth(), pbuffer->GetHeight(), GL_RGB, GL_FLOAT, m_data);
		pbuffer->EndCaptureWithoutCopy();
	}

	// draw the geometry
	void Draw(GLenum mode, int count, short *indices)
	{
//		glFlushPixelDataRangeNV(GL_READ_PIXEL_DATA_RANGE_NV); // make sure read has finished

//		glEnableClientState(GL_PRIMITIVE_RESTART_NV);
		glDrawElements(mode, count, GL_UNSIGNED_SHORT, indices);
//		glDisableClientState(GL_PRIMITIVE_RESTART_NV);
	}

	// draw the geometry
	void DrawArrays(GLenum mode, int first, int count)
	{
//		glFlushPixelDataRangeNV(GL_READ_PIXEL_DATA_RANGE_NV); // make sure read has finished

//		glEnableClientState(GL_PRIMITIVE_RESTART_NV);
		glDrawArrays(mode, first, count);
//		glDisableClientState(GL_PRIMITIVE_RESTART_NV);
	}

	char *m_data;
	unsigned int m_size;
	unsigned int m_nverts;
	unsigned int m_nattribs;
};

#endif
