// Copyright (c) 2004-2005 Lutz Latta
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.

#ifndef SimpleMesh_H
#define SimpleMesh_H

#include <vector>
#include <string>

#include <Cg/cgGL.h>
#include <glh/glh_array.h>
#include "VectorMath.h"

using namespace std;
using namespace glh;

class SimpleMesh
{

  struct Vertex
  {
    float3 Position;
    float3 Normal;
  };

  vector<Vertex> Vertices;
  vector<unsigned short> Indices;

  GLuint VertexBuffer;
  GLuint IndexBuffer;

  void UploadBuffers();

public:

  SimpleMesh();
  ~SimpleMesh();

  // Load/save mesh data.
  void Load(string const& fileName);
  void Save(string const& fileName);

  // Transform vertices permanently by 4x4 matrix.
  void TransformVertices(float4x4 const& transformation);

  // Normalize bounding box of mesh to [-1,1]^3.
  void NormalizeSizeAndCenter(void);

  // Generate mesh normals.
  void GenerateNormals(void);

  // Create mesh from height field.
  void CreateFromHeightField(const array2<unsigned char>& heightField);

  // Render mesh using weights
  void Render();

};

#endif
